@@ -1,12 +1,12 @@
 package Crypt::ECDSA::Curve::Prime;
 
-our $VERSION = '0.060';
+our $VERSION = '0.062';
 
 use base Crypt::ECDSA::Curve;
 
 use strict;
 use warnings;
-use Carp  qw( carp croak );
+use Carp qw( carp croak );
 use POSIX qw( ceil );
 
 use Crypt::ECDSA::Point;
@@ -18,25 +18,58 @@ sub is_on_curve {
     my ( $self, $x, $y ) = @_;
     return if $x == 0 and $y == 0;
     return 1
-      if ( $y * $y - ( $x * $x * $x + $self->{a} * $x + $self->{b} ) ) %
-      $self->{p} == 0;
+      if ( $y * $y - ( $x * $x * $x + $self->{a} * $x + $self->{b} ) )
+      % $self->{p} == 0;
     return;
 }
 
+sub double_on_curve {
+    my ( $self, $x, $y, $order ) = @_;
+
+    my $p = bint( $self->{p} );
+    my $a = bint( $self->{a} );
+    
+    # we cannot pass the sign easily to XS so $a_neg passes the sign of a
+    my $a_neg = bint( ( $a < 0 ) ? 1 : 0 );
+    my $x_sum = bint($x)->bmod($p);
+    my $y_sum = bint($y)->bmod($p);
+
+    Crypt::ECDSA::double_Fp_point( 
+      $x_sum->{value}, $y_sum->{value},
+      $p->{value}, $a->{value}, $a_neg->{value}
+    );
+    
+    return $self->infinity if 
+      $x_sum->is_nan or $y_sum->is_nan or 
+        ( $x_sum->is_zero and $y_sum->is_zero );
+        
+    return Crypt::ECDSA::Point->new(
+        X     => $x_sum,
+        Y     => $y_sum,
+        order => $order,
+        curve => $self
+    );
+}
+
 sub add_on_curve {
     my ( $self, $x1, $y1, $x2, $y2, $order ) = @_;
-    my $p     = bint( $self->{p} );
-    my $dy2y1 = bint( $y2 - $y1 );
-    my $dx2x1 = bint( $x2 - $x1 );
-    if ( $x1 == $x2 ) {
-        return $self->infinity if ( $y1 + $y2 ) % $p == 0;
-        return $self->double_on_curve( $x1, $y1, $order );
-    }
-    $dx2x1->bmodinv($p);
-    my $lm    = bint( $dy2y1 * $dx2x1 % $p );
-    my $x_sum = bint( ( $lm * $lm - $x1 - $x2 ) % $p );
-    my $y_sum = bint( ( $lm * ( $x1 - $x_sum ) - $y1 ) % $p );
-    return $self->infinity if $x_sum->is_nan or $y_sum->is_nan;
+    
+    my $p = bint( $self->{p} );
+    my $a = bint( $self->{a} );
+    my $a_neg = bint( ( $a < 0 ) ? 1 : 0 );
+    my $x_sum = bint($x1)->bmod($p);
+    my $y_sum = bint($y1)->bmod($p);
+    my $x2_mod = $x2 % $p;
+    my $y2_mod = $y2 % $p;
+ 
+    Crypt::ECDSA::add_Fp_point( 
+      $x_sum->{value}, $y_sum->{value}, 
+      $x2_mod->{value}, $y2_mod->{value},
+      $p->{value}, $a->{value}, $a_neg->{value}
+    );
+    return $self->infinity if 
+      $x_sum->is_nan or $y_sum->is_nan or 
+        ( $x_sum->is_zero and $y_sum->is_zero );
     return Crypt::ECDSA::Point->new(
         X     => $x_sum,
         Y     => $y_sum,
@@ -51,30 +84,16 @@ sub subtract_on_curve {
         $order );
 }
 
-sub double_on_curve {
-    my ( $self, $x, $y, $order ) = @_;
-    my $p        = $self->{p};
-    my $a        = $self->{a};
-    my $double_y = bint( $y * 2 );
-    $double_y->bmodinv($p);
-    my $lm    = ( ( 3 * $x * $x + $a ) * $double_y ) % $p;
-    my $x_sum = ( $lm * $lm - 2 * $x ) % $p;
-    my $y_sum = ( $lm * ( $x - $x_sum ) - $y ) % $p;
-    return $self->infinity if $x_sum->is_nan or $y_sum->is_nan;
-    return Crypt::ECDSA::Point->new(
-        X     => $x_sum,
-        Y     => $y_sum,
-        order => $order,
-        curve => $self
-    );
-}
 
 sub inverse_on_curve {
     my ( $self, $x, $y, $order ) = @_;
-    my $p = $self->{p};
+    my $p = bint( $self->{p} );
+    my $y_neg = bint($y);
+    $y_neg->bneg;
+    $y_neg->bmod($p);
     return Crypt::ECDSA::Point->new(
-        X     => $x,
-        Y     => ( -$y ) % $p,
+        X     => bint($x),
+        Y     => $y_neg,
         order => $order,
         curve => $self
     );
@@ -82,36 +101,33 @@ sub inverse_on_curve {
 
 sub multiply_on_curve {
     my ( $self, $x, $y, $scalar, $order ) = @_;
+    
+    my $p = bint( $self->{p} );
+    my $a = bint( $self->{a} );
+    my $a_neg = bint( ( $a < 0 ) ? 1 : 0 );
+    my $new_x = bint($x)->bmod($p);
+    my $new_y = bint($y)->bmod($p);
+    $scalar = bint($scalar);
+    $scalar->bmod($order) if $order;
+    
     croak("cannot multiply by a negative number") if $scalar < 0;
     return $self->infinity unless $scalar;
-    my $base_point = Crypt::ECDSA::Point->new(
-        X     => $x,
-        Y     => $y,
-        order => $order,
-        curve => $self
-    );
-    return $base_point if $scalar == 1;
-    my $product        = $base_point;
-    my $inverse_y_self = Crypt::ECDSA::Point->new(
-        X     => $x,
-        Y     => -$y,
-        order => $order,
-        curve => $self
-    );
-    my $tripled   = bint( $scalar * 3 );
-    my $pivot_bit = bint(1);
-    do { $pivot_bit *= 2 } while ( $pivot_bit <= $tripled );
-    $pivot_bit /= 4;
-
-    while ( $pivot_bit > 1 ) {
-        $product = $product->double();
-        my $test_tripled = $tripled & $pivot_bit;
-        my $test_scalar  = $scalar & $pivot_bit;
-        $product += $base_point     if $test_tripled and !$test_scalar;
-        $product += $inverse_y_self if $test_scalar  and !$test_tripled;
-        $pivot_bit /= 2;
+    
+    if ( $scalar != 1 ) {
+        Crypt::ECDSA::multiply_Fp_point(
+          $new_x->{value}, $new_y->{value}, $scalar->{value}, 
+          $p->{value}, $a->{value}, $a_neg->{value}
+        );
     }
-    return $product;
+    return $self->infinity 
+      if ( $new_x->is_nan or $new_y->is_nan 
+        or ( $new_x->is_zero and $new_y->is_zero ) );
+    return Crypt::ECDSA::Point->new(
+        X     => $new_x,
+        Y     => $new_y,
+        order => $order,
+        curve => $self
+    );
 }
 
 sub is_weak_curve {
@@ -174,21 +190,21 @@ sub from_octet {
     my $q_bytes       = ceil( ( length( $self->{q}->as_bin ) - 2 ) / 8 );
     my $oct_len       = length $octet;
     my $invalid_point = 1;
-    my $x = bint();
-    my $y = bint();
+    my $x             = bint();
+    my $y             = bint();
     if ( $oct_len == $q_bytes + 1 ) {    # compressed point
         my $y_byte = substr( $octet, 0, 1 );
         my $y_test;
         $y_test = 0 if $y_byte eq "\x02";
         $y_test = 1 if $y_byte eq "\x03";
-        my $x   = hex_bint( unpack( 'H*', substr( $octet, 1 ) ) );
+        my $x = hex_bint( unpack( 'H*', substr( $octet, 1 ) ) );
         if ( $x >= 0 and $x < $self->{p} and defined $y_test ) {
             my $alpha = bint();
             $alpha =
               ( $x * $x * $x + $self->{a} * $x + $self->{b} ) % $self->{p};
             $alpha->bsqrt();
             if ( ( $alpha & 1 ) == $y_test ) {
-                $y = bint( $alpha );
+                $y = bint($alpha);
             }
             else {
                 $y = bint( $self->{p} - $alpha );
@@ -205,10 +221,10 @@ sub from_octet {
             $y = hex_bint( unpack( "H*", $y_bytes ) );
             $invalid_point = 0
               if $y >= 0
-              and $y < $self->{p}
-              and $x >= 0
-              and $x < $self->{p}
-              and $self->is_on_curve($x, $y);              
+                  and $y < $self->{p}
+                  and $x >= 0
+                  and $x < $self->{p}
+                  and $self->is_on_curve( $x, $y );
         }
     }
     if ($invalid_point) {
